/******************************************************************************

  (c) 2009-2011 Scientific Computation Research Center,
      Rensselaer Polytechnic Institute. All rights reserved.

  The LICENSE-SCOREC file included with this distribution describes the terms
  of the SCOREC Non-Commercial License this program is distributed under.

*******************************************************************************/

#ifndef _H_MSG_BUFFER_HANDLE
#define _H_MSG_BUFFER_HANDLE

#include <string.h>
#include <vector>
#include <stdlib.h>

#define BUF_FULL 0
#define BUF_READ 0
#define BUF_EMPTY -1
#define NO_BUFFERS -1
#define INT_SIZE 4

class MSG_Buffer_Handle
{
  private:
    const int max_size;

    /// Current buffer size and number of messages
    int buf_size, num_blocks;

    /// Current number of messages and number of buffers
    int cur_num_blocks, num_buffers;

    /// Number of buffers used in the buffer ring and number of current buffer in the buffer container
    int num_active_buffers, cur_num_buffer;

    /// The total number of buffers which is received from the last buffer
    int final_num_buffers;

    /// Number of buffer which can be reused as all the data has been read from it
    int replace_num_buffer;

    /// The difference between active number of buffers and buffers which were not processed yet, leaving the number
    /// of buffers which can be reused 
    int read_num_buf_diff;

    /// Working buffer and current pointer in the buffer
    void *working_buf, *cur_buf;

    /// Container of buffers
    std::vector<void *> buffers;
    std::vector<void *>::iterator it_buffers;
  
  public:
    /// Allocate buffer
    void alloc();

    /// Find available buffer which can be used for putting messages in 
    void get_buffer_to_send(int rep_num_buffer);

    /// Attach message to the buffer
    int attach_data(void* attach_buf, int size_attach_buf, int fixed_size);

    /// Extract the message from the buffer
    int get_data(void *&ret_buf, int fixed_size);

    /// Finalize the last buffer 
    void finalize_buffer();

    /// Find available buffer which can be used for receiving package of messages
    void *get_buffer_to_recv();

    /// Try to get the next available buffer to receive a package
    int get_next_rcvd_buffer(int rep_num_buffer);

    /// Assign working buffer based on the number of available buffer
    void assign_working_buffer(int rep_num_buffer);

//    void take_next_buffer();
//    int take_next_sent_buffer();

    /// Get the current size of the buffer
    inline int get_buf_size(){ return buf_size; }

    /// Get the pointer to the working buffer
    inline void *get_buffer(){ return working_buf; }

    /// Assign working buffer
    inline void set_working_buffer(void *attach_buf){ working_buf = attach_buf; }

    /// Get the first buffer
    inline void *get_first_buffer(){ return buffers.front(); }

    /// Get teh number of buffers
    inline int get_num_buffers(){ return num_buffers; }

    /// Get the number of active buffers in the ring of buffers
    inline int get_num_active_buffers(){ return num_active_buffers; }

    /// Get the final number of buffers which is extracted from the last buffer
    inline int get_final_num_buffers(){ return final_num_buffers; }

    /// Get the current buffer number
    inline int get_cur_num_buffer(){ return cur_num_buffer; }

/*
    inline void MSG_Buffer_Handle::replace_buffer(void *buf, int size)
    {
      memcpy(buffers[replace_num_buffer], buf, size);
    }
*/

    /// Wrap the buffer with two numbers at the beginning of the buffer - number of messages and number of buffers
    inline void wrap_buffer(int n_buffers)
    { 
      *(int*)working_buf = num_blocks;
      *((int*)working_buf+1) = n_buffers;
    }

/*
    inline void remove_cur_buffer()
    {
      free(buffers[cur_num_buffer]); 
      buffers.erase(buffers.begin() + cur_num_buffer);
    }
*/

    /// Constructor
    MSG_Buffer_Handle(int max_buf_size):
      max_size(max_buf_size), cur_num_blocks(0), num_buffers(0), num_active_buffers(0), cur_num_buffer(0), final_num_buffers(0), replace_num_buffer(0),
      read_num_buf_diff(0){}

    /// Destructor
    ~MSG_Buffer_Handle()
    {
      for(it_buffers = buffers.begin(); it_buffers != buffers.end(); it_buffers++) 
        free(*it_buffers);
      buffers.clear();
    }
};

#endif
