/******************************************************************************

  (c) 2009-2011 Scientific Computation Research Center,
      Rensselaer Polytechnic Institute. All rights reserved.

  The LICENSE-SCOREC file included with this distribution describes the terms
  of the SCOREC Non-Commercial License this program is distributed under.

*******************************************************************************/

#include <set>
#include <cstddef>
#include <cstdlib>
#include <iostream>

#include <IPComMan.h>

typedef IPComMan comm_t;

int main(int argc, char**argv)
{
  if (argc != 3) { std::cout << "usage: " << argv[0] << " <iterations> <buffer-size>\n"; }
  int ITERATIONS =  atoi(argv[1]);
  int BUFFER_SIZE = atoi(argv[2]);
  int rank;
  int size;
  MPI_Init(NULL, NULL);
  MPI_Comm_rank(MPI_COMM_WORLD, &rank);
  MPI_Comm_size(MPI_COMM_WORLD, &size);

  comm_t communicator(MPI_COMM_WORLD,0,BUFFER_SIZE,comm_t::Neighbors);

  int to = (rank + 1) % size;
  int from = ((rank + size) - 1) % size;
  std::set<int> neighbors;
  neighbors.insert(to);
  neighbors.insert(from);
  communicator.set_Nbrs(neighbors);

  communicator.set_fixed_msg_size(sizeof(double));

  double timer = MPI_Wtime();

  double message = 3.0;
  for (int i=0; i < ITERATIONS; ++i)
  {
    communicator.send_to_nbrs(&message);
  }

  communicator.finalize_send();

  void* msg;
  int this_from;
  while (int size = communicator.receive(msg,&this_from))
  {
    free(msg);
  }

  MPI_Barrier(MPI_COMM_WORLD);
  timer = MPI_Wtime() - timer;

  if (rank == 0)
    std::cout << "Time spent: " << timer << "sec\n";

  MPI_Finalize();
  return 0;
}
