/******************************************************************************

  (c) 2009-2011 Scientific Computation Research Center,
      Rensselaer Polytechnic Institute. All rights reserved.

  The LICENSE-SCOREC file included with this distribution describes the terms
  of the SCOREC Non-Commercial License this program is distributed under.

*******************************************************************************/

#include <iostream>
#include <cstdlib>
#include <mpi.h>

int main(int argc, char**argv)
{
  if (argc != 3) { std::cout << "usage: " << argv[0] << " <iterations> <tag>\n"; }
  int iterations =  atoi(argv[1]);
  int TAG =         atoi(argv[2]);
  int rank;
  int size;
  int to, from;
  int i;
  double *message_send = new double[iterations*2];
  double *message_recv = new double[iterations*2];
  MPI_Init(NULL,NULL);
  MPI_Comm_rank(MPI_COMM_WORLD,&rank);
  MPI_Comm_size(MPI_COMM_WORLD,&size);
  MPI_Request* reqs_send = new MPI_Request[iterations*2];
  MPI_Request* reqs_recv = new MPI_Request[iterations*2];

  to = (rank + 1) % size;
  from = ((rank + size) - 1) % size;

  double timer = MPI_Wtime();

  for (i=0; i < iterations; ++i)
  {
    MPI_Isend(&message_send[2*i], 1, MPI_DOUBLE, to, TAG, MPI_COMM_WORLD, reqs_send + 2*i);
    MPI_Isend(&message_send[2*i+1], 1, MPI_DOUBLE, from, TAG, MPI_COMM_WORLD, reqs_send + 2*i + 1);
  }

  for (i=0; i < iterations; ++i)
  {
    MPI_Irecv(&message_recv[2*i], 1, MPI_DOUBLE, to, TAG, MPI_COMM_WORLD, reqs_recv + 2*i);
    MPI_Irecv(&message_recv[2*i+1], 1, MPI_DOUBLE, from, TAG, MPI_COMM_WORLD, reqs_recv + 2*i + 1);
  }


  double timer_before_waitall = MPI_Wtime() - timer;

  MPI_Waitall(iterations*2, reqs_send, MPI_STATUSES_IGNORE);
  MPI_Waitall(iterations*2, reqs_recv, MPI_STATUSES_IGNORE);

  double timer_before_reqs_delete = MPI_Wtime() - timer;

  delete [] reqs_send;
  delete [] reqs_recv;

  double timer_before_barrier = MPI_Wtime() - timer;

  MPI_Barrier(MPI_COMM_WORLD);
  timer = MPI_Wtime() - timer;

  if (rank == 0)
  {
    std::cout << "Time before waitall spent: " << timer_before_waitall << "sec\n";
    std::cout << "Time before reqs delete: " << timer_before_reqs_delete << "sec\n";
    std::cout << "Time before barrier: " << timer_before_barrier << "sec\n";
    std::cout << "Time spent: " << timer << "sec\n";
  }

  MPI_Finalize();
  return 0;
}
