/******************************************************************************

  (c) 2009-2011 Scientific Computation Research Center,
      Rensselaer Polytechnic Institute. All rights reserved.

  The LICENSE-SCOREC file included with this distribution describes the terms
  of the SCOREC Non-Commercial License this program is distributed under.

*******************************************************************************/

#include <tile.h>

#ifdef DEBUG
void print(int my_id,std::set<int>& ids)
{
  std::cerr << "neighborhood for " << my_id << " is {";
  std::set<int>::iterator it = ids.begin();
  std::set<int>::iterator end = ids.end();
  for (; it != end; ++it)
  {
    std::cerr << *it << ',';
  }
  std::cerr << "}\n";
}
#endif

void HardCodedTilePattern1(TilePattern& pattern)
{
  pattern = TilePattern(4);

  /// Adding neighbors for (0,0)
  for (int i = -1; i < 4; ++i)
  {
    for (int j = -1; j < 4; ++j)
    {
      if ((i == j && i != 0) || (i == 0 && j != 0) || (j == 0 && i != 0))
      {
        pattern.AddNeighbor(GL(0,0),GL(i,j));
      }
    }
  }
  pattern.AddNeighbor(GL(0,0),GL(-1,3));
  pattern.AddNeighbor(GL(0,0),GL(3,-1));

  /// Adding neighbors for (1,1)
  for (int i = 0; i < 3; ++i)
  {
    for (int j = 0; j < 3; ++j)
    {
      if (!(i == 1 && j == 1))
      {
        pattern.AddNeighbor(GL(1,1),GL(i,j));
      }
    }
  }
  pattern.AddNeighbor(GL(1,1),GL(3,3));

  /// Adding neighbors for (2,2)
  pattern.AddNeighbor(GL(2,2),GL(1,1));
  pattern.AddNeighbor(GL(2,2),GL(2,1));
  pattern.AddNeighbor(GL(2,2),GL(1,2));
  pattern.AddNeighbor(GL(2,2),GL(0,0));
  pattern.AddNeighbor(GL(2,2),GL(3,3));

  /// Adding neighbors for (3,3)
  for (int i = -1; i < 5; ++i)
  {
    for (int j = -1; j < 5; ++j)
    {
      if (!(i == 3 && j == 3))
      {
        pattern.AddNeighbor(GL(3,3),GL(i,j));
      }
    }
  }
  pattern.AddNeighbor(GL(3,3),GL(7,3));
  pattern.AddNeighbor(GL(3,3),GL(3,7));
  pattern.AddNeighbor(GL(3,3),GL(7,7));
  pattern.RemoveNeighbor(GL(3,3),GL(4,-1));
  pattern.RemoveNeighbor(GL(3,3),GL(-1,4));


  for (int i = 0; i < 4; ++i)
  {
    for (int j = 0; j < 4; ++j)
    {
      if (i != j)
      {
        if (i == 0 || j == 0)
        {
          pattern.AddNeighbor(GL(i,j),GL(0,0));
          if (i == 0)
          {
            pattern.AddNeighbor(GL(i,j),GL(-1,3));
          } 
          if (j == 0)
          {
            pattern.AddNeighbor(GL(i,j),GL(3,-1));
          } 
        }
        if (i == 3)
        {
          pattern.AddNeighbor(GL(i,j),GL(7,3));
        }
        if (j == 3)
        {
          pattern.AddNeighbor(GL(i,j),GL(3,7));
        }
        pattern.AddNeighbor(GL(i,j),GL(3,3));
      }
    }
   
  }


  pattern.AddNeighbor(GL(1,0),GL(1,1));
  pattern.AddNeighbor(GL(0,1),GL(1,1));
  pattern.AddNeighbor(GL(2,0),GL(1,1));
  pattern.AddNeighbor(GL(0,2),GL(1,1));
  pattern.AddNeighbor(GL(1,2),GL(1,1));
  pattern.AddNeighbor(GL(2,1),GL(1,1));

  pattern.AddNeighbor(GL(1,2),GL(2,2));
  pattern.AddNeighbor(GL(2,1),GL(2,2));

  pattern.AddNeighbor(GL(3,0),GL(4,0));
  pattern.AddNeighbor(GL(0,3),GL(0,4));
}

void CartesianTiling(TilePattern& pattern, int size)
{
  pattern = TilePattern(size);
  GridLocation owner;
  GridLocation neighbor;
  for (; owner.x < size; ++owner.x)
  {
  for (; owner.y < size; ++owner.y)
  {
  for (neighbor.x = owner.x - 1; neighbor.x < owner.x + 1; ++neighbor.x)
  {
  for (neighbor.y = owner.y - 1; neighbor.y < owner.y + 1; ++neighbor.y)
  {
    if (owner == neighbor) {continue;}
    pattern.AddNeighbor(owner, neighbor);
  }
  }
  }
  }
}

void FullyConnectedTile_SingleBoundary(TilePattern& pattern, int size)
{
  pattern = TilePattern(size);
  GridLocation owner;
  GridLocation neighbor;
  for (; owner.x < size; ++owner.x)
  {
  for (; owner.y < size; ++owner.y)
  {
  for (; neighbor.x < size; ++neighbor.x)
  {
  for (; neighbor.y < size; ++neighbor.y)
  {
    if (owner == neighbor) {continue;}
    pattern.AddNeighbor(owner, neighbor);
  }
  }
  }
  }
  for (; owner.x < size; ++owner.x)
  {
    neighbor.x = owner.x;
    owner.y = 0;
    neighbor.y = -1;
    pattern.AddNeighbor(owner, neighbor);
    owner.y = (size-1);
    neighbor.y = size;
    pattern.AddNeighbor(owner, neighbor);
  }
  for (; owner.y < size; ++owner.y)
  {
    neighbor.y = owner.y;
    owner.x = 0;
    neighbor.x = -1;
    pattern.AddNeighbor(owner, neighbor);
    owner.x = (size-1);
    neighbor.x = size;
    pattern.AddNeighbor(owner, neighbor);
  }
}

